@@ -2,6 +2,10 @@ name = "otools"
 
 __all__ = []
 
+from . import macros
+__all__.extend(macros.__all__)
+from .macros import *
+
 from . import logging
 __all__.extend(logging.__all__)
 from .logging import *
